import static com.sun.btrace.BTraceUtils.addToAggregation;
import static com.sun.btrace.BTraceUtils.newAggregation;
import static com.sun.btrace.BTraceUtils.newAggregationKey;
import static com.sun.btrace.BTraceUtils.printAggregation;
import static com.sun.btrace.BTraceUtils.println;
import static com.sun.btrace.BTraceUtils.truncateAggregation;

import com.sun.btrace.aggregation.Aggregation;
import com.sun.btrace.aggregation.AggregationFunction;
import com.sun.btrace.aggregation.AggregationKey;
import com.sun.btrace.annotations.BTrace;
import com.sun.btrace.annotations.Duration;
import com.sun.btrace.annotations.Kind;
import com.sun.btrace.annotations.Location;
import com.sun.btrace.annotations.OnEvent;
import com.sun.btrace.annotations.OnTimer;
import com.sun.btrace.annotations.OnMethod;
import com.sun.btrace.annotations.ProbeClassName;
import com.sun.btrace.annotations.ProbeMethodName;

@BTrace
public class CollectionAggregateBTrace {
    private static Aggregation histogram = newAggregation(AggregationFunction.QUANTIZE);
    private static Aggregation average = newAggregation(AggregationFunction.AVERAGE);
    private static Aggregation max = newAggregation(AggregationFunction.MAXIMUM);
    private static Aggregation min = newAggregation(AggregationFunction.MINIMUM);
    private static Aggregation sum = newAggregation(AggregationFunction.SUM);
    private static Aggregation count = newAggregation(AggregationFunction.COUNT);
    private static Aggregation globalCount = newAggregation(AggregationFunction.COUNT);

    @OnMethod(
		clazz = "/java.util.*/", 
		method = "<init>",
		location=@Location(Kind.RETURN)
	)
	public static void func(@ProbeClassName String className,
			@ProbeMethodName String methodName,@Duration long duration,int size) {
        AggregationKey key = newAggregationKey(className);

        addToAggregation(histogram, key, duration);
        addToAggregation(average, key, duration);
        addToAggregation(max, key, duration);
        addToAggregation(min, key, duration);
        addToAggregation(sum, key, duration);
        addToAggregation(count, key, duration);
        addToAggregation(globalCount, duration);
	}

	@OnTimer(10000)
   // @OnEvent
    public static void onEvent() {
        // Top 10 queries only
        truncateAggregation(histogram, 10);
        println("---------------------------------------------");
        printAggregation("Count", count);
        printAggregation("Min", min);
        printAggregation("Max", max);
        printAggregation("Average", average);
        printAggregation("Sum", sum);
        printAggregation("Histogram", histogram);
        printAggregation("Global Count", globalCount);
        println("---------------------------------------------");
    }
}

